"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _QueryStatus:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _QueryStatusEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_QueryStatus.ValueType], builtins.type):  # noqa: F821
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    INVALID: _QueryStatus.ValueType  # 0
    """Invalid status"""
    SUCCESS: _QueryStatus.ValueType  # 1
    """Query Success"""
    PENDING: _QueryStatus.ValueType  # 2
    """Query Queued"""
    CANCELED: _QueryStatus.ValueType  # 3
    """Query Canceled"""
    FAILED: _QueryStatus.ValueType  # 4
    """Query Failed"""

class QueryStatus(_QueryStatus, metaclass=_QueryStatusEnumTypeWrapper): ...

INVALID: QueryStatus.ValueType  # 0
"""Invalid status"""
SUCCESS: QueryStatus.ValueType  # 1
"""Query Success"""
PENDING: QueryStatus.ValueType  # 2
"""Query Queued"""
CANCELED: QueryStatus.ValueType  # 3
"""Query Canceled"""
FAILED: QueryStatus.ValueType  # 4
"""Query Failed"""
global___QueryStatus = QueryStatus

class _ColumnType:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _ColumnTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_ColumnType.ValueType], builtins.type):  # noqa: F821
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    INVALID_TYPE: _ColumnType.ValueType  # 0
    BOOLEAN: _ColumnType.ValueType  # 1
    """Values are true and false."""
    TINYINT: _ColumnType.ValueType  # 2
    """ A 8-bit signed integer in two’s complement format, with a minimum value of -27 and a maximum value of 27-1."""
    SMALLINT: _ColumnType.ValueType  # 3
    """A 16-bit signed integer in two’s complement format, with a minimum value of -215 and a maximum value of 215-1."""
    INT: _ColumnType.ValueType  # 4
    """INTEGER – number"""
    BIGINT: _ColumnType.ValueType  # 5
    """A 64-bit signed integer in two’s complement format, with a minimum value of -263 and a maximum value of 263-1."""
    DOUBLE: _ColumnType.ValueType  # 6
    """ A 64-bit signed double-precision floating point number."""
    FLOAT: _ColumnType.ValueType  # 7
    """ A 32-bit signed single-precision floating point number."""
    CHAR: _ColumnType.ValueType  # 8
    """Fixed length character data"""
    STRING: _ColumnType.ValueType  # 9
    """A string literal enclosed in single or double quotes"""
    BINARY: _ColumnType.ValueType  # 10
    """Binary data"""
    DATE: _ColumnType.ValueType  # 11
    """A date in ISO format, such as YYYY-MM-DD."""
    TIMESTAMP: _ColumnType.ValueType  # 12
    """Date and time instant in a java.sql.Timestamp"""

class ColumnType(_ColumnType, metaclass=_ColumnTypeEnumTypeWrapper): ...

INVALID_TYPE: ColumnType.ValueType  # 0
BOOLEAN: ColumnType.ValueType  # 1
"""Values are true and false."""
TINYINT: ColumnType.ValueType  # 2
""" A 8-bit signed integer in two’s complement format, with a minimum value of -27 and a maximum value of 27-1."""
SMALLINT: ColumnType.ValueType  # 3
"""A 16-bit signed integer in two’s complement format, with a minimum value of -215 and a maximum value of 215-1."""
INT: ColumnType.ValueType  # 4
"""INTEGER – number"""
BIGINT: ColumnType.ValueType  # 5
"""A 64-bit signed integer in two’s complement format, with a minimum value of -263 and a maximum value of 263-1."""
DOUBLE: ColumnType.ValueType  # 6
""" A 64-bit signed double-precision floating point number."""
FLOAT: ColumnType.ValueType  # 7
""" A 32-bit signed single-precision floating point number."""
CHAR: ColumnType.ValueType  # 8
"""Fixed length character data"""
STRING: ColumnType.ValueType  # 9
"""A string literal enclosed in single or double quotes"""
BINARY: ColumnType.ValueType  # 10
"""Binary data"""
DATE: ColumnType.ValueType  # 11
"""A date in ISO format, such as YYYY-MM-DD."""
TIMESTAMP: ColumnType.ValueType  # 12
"""Date and time instant in a java.sql.Timestamp"""
global___ColumnType = ColumnType

class ResultData(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    HEADERS_FIELD_NUMBER: builtins.int
    HEADERSDESC_FIELD_NUMBER: builtins.int
    ROWS_FIELD_NUMBER: builtins.int
    @property
    def headers(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """Headers of the result set"""
    @property
    def headersDesc(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___HeaderDesc]:
        """headers description"""
    @property
    def rows(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ResultRow]:
        """Rows of the result set"""
    def __init__(
        self,
        *,
        headers: collections.abc.Iterable[builtins.str] | None = ...,
        headersDesc: collections.abc.Iterable[global___HeaderDesc] | None = ...,
        rows: collections.abc.Iterable[global___ResultRow] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["headers", b"headers", "headersDesc", b"headersDesc", "rows", b"rows"]) -> None: ...

global___ResultData = ResultData

class HeaderDesc(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    TYPE_FIELD_NUMBER: builtins.int
    name: builtins.str
    """Header name"""
    type: builtins.str
    """Header type"""
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        type: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["name", b"name", "type", b"type"]) -> None: ...

global___HeaderDesc = HeaderDesc

class ResultRow(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VALUES_FIELD_NUMBER: builtins.int
    @property
    def values(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """The values in the result set row"""
    def __init__(
        self,
        *,
        values: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["values", b"values"]) -> None: ...

global___ResultRow = ResultRow

class TableColumn(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    TYPE_FIELD_NUMBER: builtins.int
    COMMENT_FIELD_NUMBER: builtins.int
    name: builtins.str
    """The name of the column"""
    type: builtins.str
    """The type of the column"""
    comment: builtins.str
    """The column's comment"""
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        type: builtins.str = ...,
        comment: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["comment", b"comment", "name", b"name", "type", b"type"]) -> None: ...

global___TableColumn = TableColumn

class TableColumnDefinition(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    TYPE_FIELD_NUMBER: builtins.int
    name: builtins.str
    """The name of the column"""
    type: global___ColumnType.ValueType
    """The type of the column"""
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        type: global___ColumnType.ValueType = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["name", b"name", "type", b"type"]) -> None: ...

global___TableColumnDefinition = TableColumnDefinition

class TableDefinition(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DB_NAME_FIELD_NUMBER: builtins.int
    TABLE_NAME_FIELD_NUMBER: builtins.int
    COLUMNS_FIELD_NUMBER: builtins.int
    DATA_PATH_FIELD_NUMBER: builtins.int
    FORMAT_FIELD_NUMBER: builtins.int
    PARTITIONS_FIELD_NUMBER: builtins.int
    db_name: builtins.str
    """The db name create table in"""
    table_name: builtins.str
    """The table name"""
    @property
    def columns(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___TableColumnDefinition]:
        """Columns to create - The order is important must be like the order in the data file."""
    data_path: builtins.str
    """Data location"""
    @property
    def format(self) -> global___DataFormat:
        """The format the data saved in"""
    @property
    def partitions(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___TableColumnDefinition]:
        """The columns to partition the data by."""
    def __init__(
        self,
        *,
        db_name: builtins.str = ...,
        table_name: builtins.str = ...,
        columns: collections.abc.Iterable[global___TableColumnDefinition] | None = ...,
        data_path: builtins.str = ...,
        format: global___DataFormat | None = ...,
        partitions: collections.abc.Iterable[global___TableColumnDefinition] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["format", b"format"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["columns", b"columns", "data_path", b"data_path", "db_name", b"db_name", "format", b"format", "partitions", b"partitions", "table_name", b"table_name"]) -> None: ...

global___TableDefinition = TableDefinition

class DataFormat(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CSV_FIELD_NUMBER: builtins.int
    @property
    def csv(self) -> global___CsvInput: ...
    def __init__(
        self,
        *,
        csv: global___CsvInput | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["csv", b"csv", "format", b"format"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["csv", b"csv", "format", b"format"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["format", b"format"]) -> typing_extensions.Literal["csv"] | None: ...

global___DataFormat = DataFormat

class CsvInput(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DELIMITER_FIELD_NUMBER: builtins.int
    QUOTE_CHAR_FIELD_NUMBER: builtins.int
    ESCAPE_CHAR_FIELD_NUMBER: builtins.int
    delimiter: builtins.str
    """Separator character - represent field in the data"""
    quote_char: builtins.str
    """Quote Character"""
    escape_char: builtins.str
    """Escape character - for special character in the data."""
    def __init__(
        self,
        *,
        delimiter: builtins.str = ...,
        quote_char: builtins.str = ...,
        escape_char: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["delimiter", b"delimiter", "escape_char", b"escape_char", "quote_char", b"quote_char"]) -> None: ...

global___CsvInput = CsvInput

class QueryResultDownloadURLParams(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    QUERY_ID_FIELD_NUMBER: builtins.int
    query_id: builtins.str
    """Query ID"""
    def __init__(
        self,
        *,
        query_id: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["query_id", b"query_id"]) -> None: ...

global___QueryResultDownloadURLParams = QueryResultDownloadURLParams



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/traffic/v1/traffic_api.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from _qwak_proto.qwak.traffic.v1 import traffic_pb2 as qwak_dot_traffic_dot_v1_dot_traffic__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n!qwak/traffic/v1/traffic_api.proto\x12\x0fqwak.traffic.v1\x1a\x1dqwak/traffic/v1/traffic.proto\"e\n\x1eResetAndSetTrafficStateRequest\x12\x43\n\x15multiple_http_traffic\x18\x01 \x01(\x0b\x32$.qwak.traffic.v1.MultipleHttpTraffic\"!\n\x1fResetAndSetTrafficStateResponse\"\x8f\x01\n\x1a\x41pplyGroupedTrafficRequest\x12\x45\n\x15multiple_http_traffic\x18\x01 \x01(\x0b\x32$.qwak.traffic.v1.MultipleHttpTrafficH\x00\x12\x10\n\x08group_id\x18\x02 \x01(\t\x12\r\n\x05hosts\x18\x03 \x03(\tB\t\n\x07traffic\"\x1d\n\x1b\x41pplyGroupedTrafficResponse\"~\n\x15\x43reateEndpointRequest\x12\x10\n\x08group_id\x18\x01 \x01(\t\x12\x10\n\x08\x65ndpoint\x18\x02 \x01(\t\x12\x11\n\tvariation\x18\x03 \x01(\t\x12\x1f\n\x17http_request_timeout_ms\x18\x04 \x01(\x05\x12\r\n\x05hosts\x18\x05 \x03(\t\"\x18\n\x16\x43reateEndpointResponse\")\n\x15\x44\x65leteEndpointRequest\x12\x10\n\x08group_id\x18\x01 \x01(\t\"\x18\n\x16\x44\x65leteEndpointResponse2\xc2\x03\n\nTrafficAPI\x12|\n\x17ResetAndSetTrafficState\x12/.qwak.traffic.v1.ResetAndSetTrafficStateRequest\x1a\x30.qwak.traffic.v1.ResetAndSetTrafficStateResponse\x12p\n\x13\x41pplyGroupedTraffic\x12+.qwak.traffic.v1.ApplyGroupedTrafficRequest\x1a,.qwak.traffic.v1.ApplyGroupedTrafficResponse\x12\x61\n\x0e\x43reateEndpoint\x12&.qwak.traffic.v1.CreateEndpointRequest\x1a\'.qwak.traffic.v1.CreateEndpointResponse\x12\x61\n\x0e\x44\x65leteEndpoint\x12&.qwak.traffic.v1.DeleteEndpointRequest\x1a\'.qwak.traffic.v1.DeleteEndpointResponseB\x17\n\x13\x63om.qwak.traffic.v1P\x01\x62\x06proto3')



_RESETANDSETTRAFFICSTATEREQUEST = DESCRIPTOR.message_types_by_name['ResetAndSetTrafficStateRequest']
_RESETANDSETTRAFFICSTATERESPONSE = DESCRIPTOR.message_types_by_name['ResetAndSetTrafficStateResponse']
_APPLYGROUPEDTRAFFICREQUEST = DESCRIPTOR.message_types_by_name['ApplyGroupedTrafficRequest']
_APPLYGROUPEDTRAFFICRESPONSE = DESCRIPTOR.message_types_by_name['ApplyGroupedTrafficResponse']
_CREATEENDPOINTREQUEST = DESCRIPTOR.message_types_by_name['CreateEndpointRequest']
_CREATEENDPOINTRESPONSE = DESCRIPTOR.message_types_by_name['CreateEndpointResponse']
_DELETEENDPOINTREQUEST = DESCRIPTOR.message_types_by_name['DeleteEndpointRequest']
_DELETEENDPOINTRESPONSE = DESCRIPTOR.message_types_by_name['DeleteEndpointResponse']
ResetAndSetTrafficStateRequest = _reflection.GeneratedProtocolMessageType('ResetAndSetTrafficStateRequest', (_message.Message,), {
  'DESCRIPTOR' : _RESETANDSETTRAFFICSTATEREQUEST,
  '__module__' : 'qwak.traffic.v1.traffic_api_pb2'
  # @@protoc_insertion_point(class_scope:qwak.traffic.v1.ResetAndSetTrafficStateRequest)
  })
_sym_db.RegisterMessage(ResetAndSetTrafficStateRequest)

ResetAndSetTrafficStateResponse = _reflection.GeneratedProtocolMessageType('ResetAndSetTrafficStateResponse', (_message.Message,), {
  'DESCRIPTOR' : _RESETANDSETTRAFFICSTATERESPONSE,
  '__module__' : 'qwak.traffic.v1.traffic_api_pb2'
  # @@protoc_insertion_point(class_scope:qwak.traffic.v1.ResetAndSetTrafficStateResponse)
  })
_sym_db.RegisterMessage(ResetAndSetTrafficStateResponse)

ApplyGroupedTrafficRequest = _reflection.GeneratedProtocolMessageType('ApplyGroupedTrafficRequest', (_message.Message,), {
  'DESCRIPTOR' : _APPLYGROUPEDTRAFFICREQUEST,
  '__module__' : 'qwak.traffic.v1.traffic_api_pb2'
  # @@protoc_insertion_point(class_scope:qwak.traffic.v1.ApplyGroupedTrafficRequest)
  })
_sym_db.RegisterMessage(ApplyGroupedTrafficRequest)

ApplyGroupedTrafficResponse = _reflection.GeneratedProtocolMessageType('ApplyGroupedTrafficResponse', (_message.Message,), {
  'DESCRIPTOR' : _APPLYGROUPEDTRAFFICRESPONSE,
  '__module__' : 'qwak.traffic.v1.traffic_api_pb2'
  # @@protoc_insertion_point(class_scope:qwak.traffic.v1.ApplyGroupedTrafficResponse)
  })
_sym_db.RegisterMessage(ApplyGroupedTrafficResponse)

CreateEndpointRequest = _reflection.GeneratedProtocolMessageType('CreateEndpointRequest', (_message.Message,), {
  'DESCRIPTOR' : _CREATEENDPOINTREQUEST,
  '__module__' : 'qwak.traffic.v1.traffic_api_pb2'
  # @@protoc_insertion_point(class_scope:qwak.traffic.v1.CreateEndpointRequest)
  })
_sym_db.RegisterMessage(CreateEndpointRequest)

CreateEndpointResponse = _reflection.GeneratedProtocolMessageType('CreateEndpointResponse', (_message.Message,), {
  'DESCRIPTOR' : _CREATEENDPOINTRESPONSE,
  '__module__' : 'qwak.traffic.v1.traffic_api_pb2'
  # @@protoc_insertion_point(class_scope:qwak.traffic.v1.CreateEndpointResponse)
  })
_sym_db.RegisterMessage(CreateEndpointResponse)

DeleteEndpointRequest = _reflection.GeneratedProtocolMessageType('DeleteEndpointRequest', (_message.Message,), {
  'DESCRIPTOR' : _DELETEENDPOINTREQUEST,
  '__module__' : 'qwak.traffic.v1.traffic_api_pb2'
  # @@protoc_insertion_point(class_scope:qwak.traffic.v1.DeleteEndpointRequest)
  })
_sym_db.RegisterMessage(DeleteEndpointRequest)

DeleteEndpointResponse = _reflection.GeneratedProtocolMessageType('DeleteEndpointResponse', (_message.Message,), {
  'DESCRIPTOR' : _DELETEENDPOINTRESPONSE,
  '__module__' : 'qwak.traffic.v1.traffic_api_pb2'
  # @@protoc_insertion_point(class_scope:qwak.traffic.v1.DeleteEndpointResponse)
  })
_sym_db.RegisterMessage(DeleteEndpointResponse)

_TRAFFICAPI = DESCRIPTOR.services_by_name['TrafficAPI']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\023com.qwak.traffic.v1P\001'
  _RESETANDSETTRAFFICSTATEREQUEST._serialized_start=85
  _RESETANDSETTRAFFICSTATEREQUEST._serialized_end=186
  _RESETANDSETTRAFFICSTATERESPONSE._serialized_start=188
  _RESETANDSETTRAFFICSTATERESPONSE._serialized_end=221
  _APPLYGROUPEDTRAFFICREQUEST._serialized_start=224
  _APPLYGROUPEDTRAFFICREQUEST._serialized_end=367
  _APPLYGROUPEDTRAFFICRESPONSE._serialized_start=369
  _APPLYGROUPEDTRAFFICRESPONSE._serialized_end=398
  _CREATEENDPOINTREQUEST._serialized_start=400
  _CREATEENDPOINTREQUEST._serialized_end=526
  _CREATEENDPOINTRESPONSE._serialized_start=528
  _CREATEENDPOINTRESPONSE._serialized_end=552
  _DELETEENDPOINTREQUEST._serialized_start=554
  _DELETEENDPOINTREQUEST._serialized_end=595
  _DELETEENDPOINTRESPONSE._serialized_start=597
  _DELETEENDPOINTRESPONSE._serialized_end=621
  _TRAFFICAPI._serialized_start=624
  _TRAFFICAPI._serialized_end=1074
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _MemoryUnit:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _MemoryUnitEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_MemoryUnit.ValueType], builtins.type):  # noqa: F821
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    INVALID_MEMORY_UNIT: _MemoryUnit.ValueType  # 0
    MIB: _MemoryUnit.ValueType  # 1
    GIB: _MemoryUnit.ValueType  # 2

class MemoryUnit(_MemoryUnit, metaclass=_MemoryUnitEnumTypeWrapper): ...

INVALID_MEMORY_UNIT: MemoryUnit.ValueType  # 0
MIB: MemoryUnit.ValueType  # 1
GIB: MemoryUnit.ValueType  # 2
global___MemoryUnit = MemoryUnit

class _GpuType:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _GpuTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_GpuType.ValueType], builtins.type):  # noqa: F821
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    INVALID_GPU_TYPE: _GpuType.ValueType  # 0
    NVIDIA_K80: _GpuType.ValueType  # 1
    NVIDIA_V100: _GpuType.ValueType  # 2
    NVIDIA_A100: _GpuType.ValueType  # 3
    NVIDIA_T4: _GpuType.ValueType  # 4
    NVIDIA_A10G: _GpuType.ValueType  # 5
    NVIDIA_L4: _GpuType.ValueType  # 6
    NVIDIA_T4_1_4_15: _GpuType.ValueType  # 7
    """More specific node types
    t4.xl
    """
    NVIDIA_T4_1_8_30: _GpuType.ValueType  # 8
    """t4.2xl"""
    NVIDIA_T4_1_16_60: _GpuType.ValueType  # 9
    """t4.4xl"""
    NVIDIA_A100_80GB_8_96_1360: _GpuType.ValueType  # 10
    """a100.8xl"""
    NVIDIA_V100_1_8_52: _GpuType.ValueType  # 11
    """a100.xl"""
    NVIDIA_V100_4_32_208: _GpuType.ValueType  # 12
    """v100.4xl"""

class GpuType(_GpuType, metaclass=_GpuTypeEnumTypeWrapper): ...

INVALID_GPU_TYPE: GpuType.ValueType  # 0
NVIDIA_K80: GpuType.ValueType  # 1
NVIDIA_V100: GpuType.ValueType  # 2
NVIDIA_A100: GpuType.ValueType  # 3
NVIDIA_T4: GpuType.ValueType  # 4
NVIDIA_A10G: GpuType.ValueType  # 5
NVIDIA_L4: GpuType.ValueType  # 6
NVIDIA_T4_1_4_15: GpuType.ValueType  # 7
"""More specific node types
t4.xl
"""
NVIDIA_T4_1_8_30: GpuType.ValueType  # 8
"""t4.2xl"""
NVIDIA_T4_1_16_60: GpuType.ValueType  # 9
"""t4.4xl"""
NVIDIA_A100_80GB_8_96_1360: GpuType.ValueType  # 10
"""a100.8xl"""
NVIDIA_V100_1_8_52: GpuType.ValueType  # 11
"""a100.xl"""
NVIDIA_V100_4_32_208: GpuType.ValueType  # 12
"""v100.4xl"""
global___GpuType = GpuType

class _SparkClusterResourceTemplate:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _SparkClusterResourceTemplateEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_SparkClusterResourceTemplate.ValueType], builtins.type):  # noqa: F821
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    SPARK_CLUSTER_RESOURCE_INVALID: _SparkClusterResourceTemplate.ValueType  # 0
    SPARK_CLUSTER_RESOURCE_SMALL: _SparkClusterResourceTemplate.ValueType  # 1
    SPARK_CLUSTER_RESOURCE_MEDIUM: _SparkClusterResourceTemplate.ValueType  # 2
    SPARK_CLUSTER_RESOURCE_LARGE: _SparkClusterResourceTemplate.ValueType  # 3
    SPARK_CLUSTER_RESOURCE_XL: _SparkClusterResourceTemplate.ValueType  # 4
    SPARK_CLUSTER_RESOURCE_2XL: _SparkClusterResourceTemplate.ValueType  # 5
    SPARK_CLUSTER_RESOURCE_3XL: _SparkClusterResourceTemplate.ValueType  # 6

class SparkClusterResourceTemplate(_SparkClusterResourceTemplate, metaclass=_SparkClusterResourceTemplateEnumTypeWrapper):
    """Templates for clusters the user may choose"""

SPARK_CLUSTER_RESOURCE_INVALID: SparkClusterResourceTemplate.ValueType  # 0
SPARK_CLUSTER_RESOURCE_SMALL: SparkClusterResourceTemplate.ValueType  # 1
SPARK_CLUSTER_RESOURCE_MEDIUM: SparkClusterResourceTemplate.ValueType  # 2
SPARK_CLUSTER_RESOURCE_LARGE: SparkClusterResourceTemplate.ValueType  # 3
SPARK_CLUSTER_RESOURCE_XL: SparkClusterResourceTemplate.ValueType  # 4
SPARK_CLUSTER_RESOURCE_2XL: SparkClusterResourceTemplate.ValueType  # 5
SPARK_CLUSTER_RESOURCE_3XL: SparkClusterResourceTemplate.ValueType  # 6
global___SparkClusterResourceTemplate = SparkClusterResourceTemplate

class _NodeOptimizationType:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _NodeOptimizationTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_NodeOptimizationType.ValueType], builtins.type):  # noqa: F821
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    NODE_OPTIMIZATION_INVALID: _NodeOptimizationType.ValueType  # 0
    NODE_OPTIMIZATION_NONE: _NodeOptimizationType.ValueType  # 1
    NODE_OPTIMIZATION_CPU: _NodeOptimizationType.ValueType  # 2
    NODE_OPTIMIZATION_MEMORY: _NodeOptimizationType.ValueType  # 3

class NodeOptimizationType(_NodeOptimizationType, metaclass=_NodeOptimizationTypeEnumTypeWrapper): ...

NODE_OPTIMIZATION_INVALID: NodeOptimizationType.ValueType  # 0
NODE_OPTIMIZATION_NONE: NodeOptimizationType.ValueType  # 1
NODE_OPTIMIZATION_CPU: NodeOptimizationType.ValueType  # 2
NODE_OPTIMIZATION_MEMORY: NodeOptimizationType.ValueType  # 3
global___NodeOptimizationType = NodeOptimizationType

class CpuResources(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CPU_FIELD_NUMBER: builtins.int
    MEMORY_AMOUNT_FIELD_NUMBER: builtins.int
    MEMORY_UNITS_FIELD_NUMBER: builtins.int
    cpu: builtins.float
    """Amount of CPU cores"""
    memory_amount: builtins.int
    """Amount of memory"""
    memory_units: global___MemoryUnit.ValueType
    """Units type of memory"""
    def __init__(
        self,
        *,
        cpu: builtins.float = ...,
        memory_amount: builtins.int = ...,
        memory_units: global___MemoryUnit.ValueType = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["cpu", b"cpu", "memory_amount", b"memory_amount", "memory_units", b"memory_units"]) -> None: ...

global___CpuResources = CpuResources

class GpuResources(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    GPU_TYPE_FIELD_NUMBER: builtins.int
    GPU_AMOUNT_FIELD_NUMBER: builtins.int
    gpu_type: global___GpuType.ValueType
    """The type of the GPU"""
    gpu_amount: builtins.int
    """Amount of GPUs"""
    def __init__(
        self,
        *,
        gpu_type: global___GpuType.ValueType = ...,
        gpu_amount: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["gpu_amount", b"gpu_amount", "gpu_type", b"gpu_type"]) -> None: ...

global___GpuResources = GpuResources

class SparkClusterResourceTemplateSpec(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TEMPLATE_FIELD_NUMBER: builtins.int
    template: global___SparkClusterResourceTemplate.ValueType
    def __init__(
        self,
        *,
        template: global___SparkClusterResourceTemplate.ValueType = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["template", b"template"]) -> None: ...

global___SparkClusterResourceTemplateSpec = SparkClusterResourceTemplateSpec

class SparkResourceConfiguration(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DRIVER_MEMORY_FIELD_NUMBER: builtins.int
    DRIVER_CORES_FIELD_NUMBER: builtins.int
    INITIAL_EXECUTORS_FIELD_NUMBER: builtins.int
    MIN_EXECUTORS_FIELD_NUMBER: builtins.int
    MAX_EXECUTORS_FIELD_NUMBER: builtins.int
    EXECUTOR_MEMORY_FIELD_NUMBER: builtins.int
    EXECUTOR_CORES_FIELD_NUMBER: builtins.int
    driver_memory: builtins.str
    """driver memory, corresponds to "spark.driver.memory" and has the same format"""
    driver_cores: builtins.int
    """driver cores, corresponds to "spark.driver.cores" """
    initial_executors: builtins.int
    """initial executor count, corresponds to "spark.dynamicAllocation.initialExecutors" """
    min_executors: builtins.int
    """min number of executors, corresponds to "spark.dynamicAllocation.minExecutors" """
    max_executors: builtins.int
    """max number of executors, corresponds to "spark.dynamicAllocation.maxExecutors" """
    executor_memory: builtins.str
    """executor memory, corresponds to (and has the same format of) "spark.executor.memory" """
    executor_cores: builtins.int
    """executor cores, corresponds to "spark.executor.cores" """
    def __init__(
        self,
        *,
        driver_memory: builtins.str = ...,
        driver_cores: builtins.int = ...,
        initial_executors: builtins.int = ...,
        min_executors: builtins.int = ...,
        max_executors: builtins.int = ...,
        executor_memory: builtins.str = ...,
        executor_cores: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["driver_cores", b"driver_cores", "driver_memory", b"driver_memory", "executor_cores", b"executor_cores", "executor_memory", b"executor_memory", "initial_executors", b"initial_executors", "max_executors", b"max_executors", "min_executors", b"min_executors"]) -> None: ...

global___SparkResourceConfiguration = SparkResourceConfiguration

class PodComputeResourceTemplateSpec(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TEMPLATE_ID_FIELD_NUMBER: builtins.int
    template_id: builtins.str
    def __init__(
        self,
        *,
        template_id: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["template_id", b"template_id"]) -> None: ...

global___PodComputeResourceTemplateSpec = PodComputeResourceTemplateSpec

class PodComputeResources(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    OPTIMIZATION_TYPE_FIELD_NUMBER: builtins.int
    CPU_RESOURCES_FIELD_NUMBER: builtins.int
    GPU_RESOURCES_FIELD_NUMBER: builtins.int
    TEMPLATE_SPEC_FIELD_NUMBER: builtins.int
    optimization_type: global___NodeOptimizationType.ValueType
    @property
    def cpu_resources(self) -> global___CpuResources: ...
    @property
    def gpu_resources(self) -> global___GpuResources: ...
    @property
    def template_spec(self) -> global___PodComputeResourceTemplateSpec: ...
    def __init__(
        self,
        *,
        optimization_type: global___NodeOptimizationType.ValueType = ...,
        cpu_resources: global___CpuResources | None = ...,
        gpu_resources: global___GpuResources | None = ...,
        template_spec: global___PodComputeResourceTemplateSpec | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["cpu_resources", b"cpu_resources", "gpu_resources", b"gpu_resources", "template_spec", b"template_spec"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["cpu_resources", b"cpu_resources", "gpu_resources", b"gpu_resources", "optimization_type", b"optimization_type", "template_spec", b"template_spec"]) -> None: ...

global___PodComputeResources = PodComputeResources

class SparkComputeResources(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    RESOURCE_CONFIGURATION_FIELD_NUMBER: builtins.int
    @property
    def resource_configuration(self) -> global___SparkResourceConfiguration: ...
    def __init__(
        self,
        *,
        resource_configuration: global___SparkResourceConfiguration | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["resource_configuration", b"resource_configuration"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["resource_configuration", b"resource_configuration"]) -> None: ...

global___SparkComputeResources = SparkComputeResources

class ClientSparkComputeResources(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TEMPLATE_SPEC_FIELD_NUMBER: builtins.int
    RESOURCE_CONFIGURATION_FIELD_NUMBER: builtins.int
    @property
    def template_spec(self) -> global___SparkClusterResourceTemplateSpec: ...
    @property
    def resource_configuration(self) -> global___SparkResourceConfiguration: ...
    def __init__(
        self,
        *,
        template_spec: global___SparkClusterResourceTemplateSpec | None = ...,
        resource_configuration: global___SparkResourceConfiguration | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["resource_configuration", b"resource_configuration", "resources", b"resources", "template_spec", b"template_spec"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["resource_configuration", b"resource_configuration", "resources", b"resources", "template_spec", b"template_spec"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["resources", b"resources"]) -> typing_extensions.Literal["template_spec", "resource_configuration"] | None: ...

global___ClientSparkComputeResources = ClientSparkComputeResources

class ClientPodComputeResources(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NODE_OPTIMIZATION_TYPE_FIELD_NUMBER: builtins.int
    TEMPLATE_SPEC_FIELD_NUMBER: builtins.int
    CPU_RESOURCES_FIELD_NUMBER: builtins.int
    GPU_RESOURCES_FIELD_NUMBER: builtins.int
    node_optimization_type: global___NodeOptimizationType.ValueType
    @property
    def template_spec(self) -> global___PodComputeResourceTemplateSpec: ...
    @property
    def cpu_resources(self) -> global___CpuResources: ...
    @property
    def gpu_resources(self) -> global___GpuResources: ...
    def __init__(
        self,
        *,
        node_optimization_type: global___NodeOptimizationType.ValueType = ...,
        template_spec: global___PodComputeResourceTemplateSpec | None = ...,
        cpu_resources: global___CpuResources | None = ...,
        gpu_resources: global___GpuResources | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["cpu_resources", b"cpu_resources", "gpu_resources", b"gpu_resources", "resources", b"resources", "template_spec", b"template_spec"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["cpu_resources", b"cpu_resources", "gpu_resources", b"gpu_resources", "node_optimization_type", b"node_optimization_type", "resources", b"resources", "template_spec", b"template_spec"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["resources", b"resources"]) -> typing_extensions.Literal["template_spec", "cpu_resources", "gpu_resources"] | None: ...

global___ClientPodComputeResources = ClientPodComputeResources



from typing import Optional


class BuildPhase:
    def __init__(
        self,
        phase_id: str,
        name: Optional[str] = None,
        description: Optional[str] = None,
    ):
        self.phase_id = phase_id
        self.name = name
        self.description = description
        if not self.name:
            self.name = phase_id.replace("_", " ").capitalize()
        if not self.description:
            self.description = phase_id.replace("_", " ").capitalize()

    def __eq__(self, other):
        return (
            self.phase_id == other.phase_id
            and self.name == other.name
            and self.description == other.description
        )



from abc import ABC, abstractmethod
from dataclasses import dataclass, field
from typing import Optional

from _qwak_proto.qwak.feature_store.v1.common.source_code.source_code_pb2 import (
    SourceCodeSpec as ProtoSourceCodeSpec,
    SourceCodeArtifact as ProtoSourceCodeArtifact,
    ZipArtifact as ProtoZipArtifact,
)
from qwak.exceptions import QwakException


class SourceCodeArtifact(ABC):
    @abstractmethod
    def _to_proto(self) -> ProtoSourceCodeArtifact:
        pass


@dataclass
class ZipArtifact(SourceCodeArtifact):
    artifact_path: str
    main_file: str

    @classmethod
    def _from_proto(cls, proto: ProtoSourceCodeArtifact) -> "ZipArtifact":
        artifact_type: str = proto.WhichOneof("type")
        if artifact_type != "zip_artifact":
            raise QwakException(f"Instead of `zip_artifact` got: {artifact_type}")

        return cls(
            artifact_path=proto.zip_artifact.path,
            main_file=proto.zip_artifact.main_file,
        )

    def _to_proto(self) -> ProtoSourceCodeArtifact:
        return ProtoSourceCodeArtifact(
            zip_artifact=ProtoZipArtifact(
                path=self.artifact_path, main_file=self.main_file
            )
        )


@dataclass
class SourceCodeSpec:
    artifact: Optional[SourceCodeArtifact] = field(default=None)

    @classmethod
    def _from_proto(cls, proto: ProtoSourceCodeSpec) -> "SourceCodeSpec":
        artifact_type: str = proto.artifact.WhichOneof("type")
        if artifact_type == "zip_artifact":
            return cls(artifact=ZipArtifact._from_proto(proto.artifact))

        return cls(artifact=None)

    def _to_proto(self) -> ProtoSourceCodeSpec:
        if self.artifact:
            return ProtoSourceCodeSpec(artifact=self.artifact._to_proto())
        return ProtoSourceCodeSpec()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/execution/v1/state/execution_state_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from _qwak_proto.qwak.execution.v1.state import execution_state_pb2 as qwak_dot_execution_dot_v1_dot_state_dot_execution__state__pb2
from _qwak_proto.qwak.execution.v1.state import featureset_state_pb2 as qwak_dot_execution_dot_v1_dot_state_dot_featureset__state__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n5qwak/execution/v1/state/execution_state_service.proto\x12%qwak.feature.store.execution.v1.state\x1a-qwak/execution/v1/state/execution_state.proto\x1a.qwak/execution/v1/state/featureset_state.proto\"6\n\x1dGetFeaturesetStateByIdRequest\x12\x15\n\rfeatureset_id\x18\x01 \x01(\t\"r\n\x1eGetFeaturesetStateByIdResponse\x12P\n\x10\x66\x65\x61tureset_state\x18\x01 \x01(\x0b\x32\x36.qwak.feature.store.execution.v1.state.FeaturesetState\":\n\x1fGetFeaturesetStateByNameRequest\x12\x17\n\x0f\x66\x65\x61tureset_name\x18\x01 \x01(\t\"t\n GetFeaturesetStateByNameResponse\x12P\n\x10\x66\x65\x61tureset_state\x18\x01 \x01(\x0b\x32\x36.qwak.feature.store.execution.v1.state.FeaturesetState\"=\n#GetMultiFeaturesetStateByIdsRequest\x12\x16\n\x0e\x66\x65\x61tureset_ids\x18\x01 \x03(\t\"y\n$GetMultiFeaturesetStateByIdsResponse\x12Q\n\x11\x66\x65\x61tureset_states\x18\x01 \x03(\x0b\x32\x36.qwak.feature.store.execution.v1.state.FeaturesetState\"?\n$GetMultiFeaturesetStateByNameRequest\x12\x17\n\x0f\x66\x65\x61tureset_name\x18\x01 \x03(\t\"y\n%GetMultiFeaturesetStateByNameResponse\x12P\n\x10\x66\x65\x61tureset_state\x18\x01 \x03(\x0b\x32\x36.qwak.feature.store.execution.v1.state.FeaturesetState\"r\n UpdateMultiExecutionStateRequest\x12N\n\x0f\x65xecution_state\x18\x01 \x03(\x0b\x32\x35.qwak.feature.store.execution.v1.state.ExecutionState\"#\n!UpdateMultiExecutionStateResponse\"9\n DeleteFeaturesetStateByIdRequest\x12\x15\n\rfeatureset_id\x18\x01 \x01(\t\"@\n!DeleteFeaturesetStateByIdResponse\x12\x1b\n\x13num_deleted_records\x18\x01 \x01(\x03\x32\xd2\x08\n!FeatureStoreExecutionStateService\x12\xa5\x01\n\x16GetFeaturesetStateById\x12\x44.qwak.feature.store.execution.v1.state.GetFeaturesetStateByIdRequest\x1a\x45.qwak.feature.store.execution.v1.state.GetFeaturesetStateByIdResponse\x12\xab\x01\n\x18GetFeaturesetStateByName\x12\x46.qwak.feature.store.execution.v1.state.GetFeaturesetStateByNameRequest\x1aG.qwak.feature.store.execution.v1.state.GetFeaturesetStateByNameResponse\x12\xb7\x01\n\x1cGetMultiFeaturesetStateByIds\x12J.qwak.feature.store.execution.v1.state.GetMultiFeaturesetStateByIdsRequest\x1aK.qwak.feature.store.execution.v1.state.GetMultiFeaturesetStateByIdsResponse\x12\xba\x01\n\x1dGetMultiFeaturesetStateByName\x12K.qwak.feature.store.execution.v1.state.GetMultiFeaturesetStateByNameRequest\x1aL.qwak.feature.store.execution.v1.state.GetMultiFeaturesetStateByNameResponse\x12\xae\x01\n\x19UpdateMultiExecutionState\x12G.qwak.feature.store.execution.v1.state.UpdateMultiExecutionStateRequest\x1aH.qwak.feature.store.execution.v1.state.UpdateMultiExecutionStateResponse\x12\xae\x01\n\x19\x44\x65leteFeaturesetStateById\x12G.qwak.feature.store.execution.v1.state.DeleteFeaturesetStateByIdRequest\x1aH.qwak.feature.store.execution.v1.state.DeleteFeaturesetStateByIdResponseB`\n+com.qwak.ai.features.execution.api.v1.stateP\x01Z/qwak/execution_management/state;execution_stateb\x06proto3')



_GETFEATURESETSTATEBYIDREQUEST = DESCRIPTOR.message_types_by_name['GetFeaturesetStateByIdRequest']
_GETFEATURESETSTATEBYIDRESPONSE = DESCRIPTOR.message_types_by_name['GetFeaturesetStateByIdResponse']
_GETFEATURESETSTATEBYNAMEREQUEST = DESCRIPTOR.message_types_by_name['GetFeaturesetStateByNameRequest']
_GETFEATURESETSTATEBYNAMERESPONSE = DESCRIPTOR.message_types_by_name['GetFeaturesetStateByNameResponse']
_GETMULTIFEATURESETSTATEBYIDSREQUEST = DESCRIPTOR.message_types_by_name['GetMultiFeaturesetStateByIdsRequest']
_GETMULTIFEATURESETSTATEBYIDSRESPONSE = DESCRIPTOR.message_types_by_name['GetMultiFeaturesetStateByIdsResponse']
_GETMULTIFEATURESETSTATEBYNAMEREQUEST = DESCRIPTOR.message_types_by_name['GetMultiFeaturesetStateByNameRequest']
_GETMULTIFEATURESETSTATEBYNAMERESPONSE = DESCRIPTOR.message_types_by_name['GetMultiFeaturesetStateByNameResponse']
_UPDATEMULTIEXECUTIONSTATEREQUEST = DESCRIPTOR.message_types_by_name['UpdateMultiExecutionStateRequest']
_UPDATEMULTIEXECUTIONSTATERESPONSE = DESCRIPTOR.message_types_by_name['UpdateMultiExecutionStateResponse']
_DELETEFEATURESETSTATEBYIDREQUEST = DESCRIPTOR.message_types_by_name['DeleteFeaturesetStateByIdRequest']
_DELETEFEATURESETSTATEBYIDRESPONSE = DESCRIPTOR.message_types_by_name['DeleteFeaturesetStateByIdResponse']
GetFeaturesetStateByIdRequest = _reflection.GeneratedProtocolMessageType('GetFeaturesetStateByIdRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETFEATURESETSTATEBYIDREQUEST,
  '__module__' : 'qwak.execution.v1.state.execution_state_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.execution.v1.state.GetFeaturesetStateByIdRequest)
  })
_sym_db.RegisterMessage(GetFeaturesetStateByIdRequest)

GetFeaturesetStateByIdResponse = _reflection.GeneratedProtocolMessageType('GetFeaturesetStateByIdResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETFEATURESETSTATEBYIDRESPONSE,
  '__module__' : 'qwak.execution.v1.state.execution_state_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.execution.v1.state.GetFeaturesetStateByIdResponse)
  })
_sym_db.RegisterMessage(GetFeaturesetStateByIdResponse)

GetFeaturesetStateByNameRequest = _reflection.GeneratedProtocolMessageType('GetFeaturesetStateByNameRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETFEATURESETSTATEBYNAMEREQUEST,
  '__module__' : 'qwak.execution.v1.state.execution_state_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.execution.v1.state.GetFeaturesetStateByNameRequest)
  })
_sym_db.RegisterMessage(GetFeaturesetStateByNameRequest)

GetFeaturesetStateByNameResponse = _reflection.GeneratedProtocolMessageType('GetFeaturesetStateByNameResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETFEATURESETSTATEBYNAMERESPONSE,
  '__module__' : 'qwak.execution.v1.state.execution_state_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.execution.v1.state.GetFeaturesetStateByNameResponse)
  })
_sym_db.RegisterMessage(GetFeaturesetStateByNameResponse)

GetMultiFeaturesetStateByIdsRequest = _reflection.GeneratedProtocolMessageType('GetMultiFeaturesetStateByIdsRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETMULTIFEATURESETSTATEBYIDSREQUEST,
  '__module__' : 'qwak.execution.v1.state.execution_state_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.execution.v1.state.GetMultiFeaturesetStateByIdsRequest)
  })
_sym_db.RegisterMessage(GetMultiFeaturesetStateByIdsRequest)

GetMultiFeaturesetStateByIdsResponse = _reflection.GeneratedProtocolMessageType('GetMultiFeaturesetStateByIdsResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETMULTIFEATURESETSTATEBYIDSRESPONSE,
  '__module__' : 'qwak.execution.v1.state.execution_state_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.execution.v1.state.GetMultiFeaturesetStateByIdsResponse)
  })
_sym_db.RegisterMessage(GetMultiFeaturesetStateByIdsResponse)

GetMultiFeaturesetStateByNameRequest = _reflection.GeneratedProtocolMessageType('GetMultiFeaturesetStateByNameRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETMULTIFEATURESETSTATEBYNAMEREQUEST,
  '__module__' : 'qwak.execution.v1.state.execution_state_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.execution.v1.state.GetMultiFeaturesetStateByNameRequest)
  })
_sym_db.RegisterMessage(GetMultiFeaturesetStateByNameRequest)

GetMultiFeaturesetStateByNameResponse = _reflection.GeneratedProtocolMessageType('GetMultiFeaturesetStateByNameResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETMULTIFEATURESETSTATEBYNAMERESPONSE,
  '__module__' : 'qwak.execution.v1.state.execution_state_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.execution.v1.state.GetMultiFeaturesetStateByNameResponse)
  })
_sym_db.RegisterMessage(GetMultiFeaturesetStateByNameResponse)

UpdateMultiExecutionStateRequest = _reflection.GeneratedProtocolMessageType('UpdateMultiExecutionStateRequest', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEMULTIEXECUTIONSTATEREQUEST,
  '__module__' : 'qwak.execution.v1.state.execution_state_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.execution.v1.state.UpdateMultiExecutionStateRequest)
  })
_sym_db.RegisterMessage(UpdateMultiExecutionStateRequest)

UpdateMultiExecutionStateResponse = _reflection.GeneratedProtocolMessageType('UpdateMultiExecutionStateResponse', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEMULTIEXECUTIONSTATERESPONSE,
  '__module__' : 'qwak.execution.v1.state.execution_state_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.execution.v1.state.UpdateMultiExecutionStateResponse)
  })
_sym_db.RegisterMessage(UpdateMultiExecutionStateResponse)

DeleteFeaturesetStateByIdRequest = _reflection.GeneratedProtocolMessageType('DeleteFeaturesetStateByIdRequest', (_message.Message,), {
  'DESCRIPTOR' : _DELETEFEATURESETSTATEBYIDREQUEST,
  '__module__' : 'qwak.execution.v1.state.execution_state_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.execution.v1.state.DeleteFeaturesetStateByIdRequest)
  })
_sym_db.RegisterMessage(DeleteFeaturesetStateByIdRequest)

DeleteFeaturesetStateByIdResponse = _reflection.GeneratedProtocolMessageType('DeleteFeaturesetStateByIdResponse', (_message.Message,), {
  'DESCRIPTOR' : _DELETEFEATURESETSTATEBYIDRESPONSE,
  '__module__' : 'qwak.execution.v1.state.execution_state_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.execution.v1.state.DeleteFeaturesetStateByIdResponse)
  })
_sym_db.RegisterMessage(DeleteFeaturesetStateByIdResponse)

_FEATURESTOREEXECUTIONSTATESERVICE = DESCRIPTOR.services_by_name['FeatureStoreExecutionStateService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n+com.qwak.ai.features.execution.api.v1.stateP\001Z/qwak/execution_management/state;execution_state'
  _GETFEATURESETSTATEBYIDREQUEST._serialized_start=191
  _GETFEATURESETSTATEBYIDREQUEST._serialized_end=245
  _GETFEATURESETSTATEBYIDRESPONSE._serialized_start=247
  _GETFEATURESETSTATEBYIDRESPONSE._serialized_end=361
  _GETFEATURESETSTATEBYNAMEREQUEST._serialized_start=363
  _GETFEATURESETSTATEBYNAMEREQUEST._serialized_end=421
  _GETFEATURESETSTATEBYNAMERESPONSE._serialized_start=423
  _GETFEATURESETSTATEBYNAMERESPONSE._serialized_end=539
  _GETMULTIFEATURESETSTATEBYIDSREQUEST._serialized_start=541
  _GETMULTIFEATURESETSTATEBYIDSREQUEST._serialized_end=602
  _GETMULTIFEATURESETSTATEBYIDSRESPONSE._serialized_start=604
  _GETMULTIFEATURESETSTATEBYIDSRESPONSE._serialized_end=725
  _GETMULTIFEATURESETSTATEBYNAMEREQUEST._serialized_start=727
  _GETMULTIFEATURESETSTATEBYNAMEREQUEST._serialized_end=790
  _GETMULTIFEATURESETSTATEBYNAMERESPONSE._serialized_start=792
  _GETMULTIFEATURESETSTATEBYNAMERESPONSE._serialized_end=913
  _UPDATEMULTIEXECUTIONSTATEREQUEST._serialized_start=915
  _UPDATEMULTIEXECUTIONSTATEREQUEST._serialized_end=1029
  _UPDATEMULTIEXECUTIONSTATERESPONSE._serialized_start=1031
  _UPDATEMULTIEXECUTIONSTATERESPONSE._serialized_end=1066
  _DELETEFEATURESETSTATEBYIDREQUEST._serialized_start=1068
  _DELETEFEATURESETSTATEBYIDREQUEST._serialized_end=1125
  _DELETEFEATURESETSTATEBYIDRESPONSE._serialized_start=1127
  _DELETEFEATURESETSTATEBYIDRESPONSE._serialized_end=1191
  _FEATURESTOREEXECUTIONSTATESERVICE._serialized_start=1194
  _FEATURESTOREEXECUTIONSTATESERVICE._serialized_end=2300
# @@protoc_insertion_point(module_scope)



import time
from dataclasses import dataclass
from typing import Dict, Optional

from _qwak_proto.qwak.build.v1.build_pb2 import (
    BatchFeature as ProtoBatchFeature,
    BatchFeatureV1 as ProtoBatchFeatureV1,
    Feature as ProtoFeature,
    StreamingAggregationFeature as ProtoStreamingAggregationFeature,
    StreamingFeature as ProtoStreamingFeature,
    StreamingFeatureV1 as ProtoStreamingFeatureV1,
)
from _qwak_proto.qwak.feature_store.entities.entity_pb2 import (
    EntitySpec as ProtoEntitySpec,
)
from _qwak_proto.qwak.feature_store.features.feature_set_types_pb2 import (
    BatchFeatureSet as ProtoBatchFeatureSet,
    BatchFeatureSetV1 as ProtoBatchFeatureSetV1,
    FeatureSetType as ProtoFeatureSetType,
    StreamingAggregationFeatureSet as ProtoStreamingAggregationFeatureSet,
    StreamingFeatureSetV1 as ProtoStreamingFeatureSetV1,
)
from qwak.clients.feature_store import FeatureRegistryClient
from qwak.exceptions import QwakException
from qwak.model.schema_entities import BaseFeature, Entity, FeatureStoreInput

_INTERNAL_KEY_PREFIX: str = "_qwak_key_"


@dataclass(unsafe_hash=True)
class BatchFeature(BaseFeature):
    entity: Optional[Entity] = None

    def to_proto(self):
        return ProtoFeature(
            batch_feature=ProtoBatchFeature(
                name=self.name, entity=self.entity.to_proto()
            )
        )


@dataclass(unsafe_hash=True)
class BatchFeatureV1(BaseFeature):
    entity: Optional[Entity] = None

    def to_proto(self):
        return ProtoFeature(
            batch_feature_v1=ProtoBatchFeatureV1(
                name=self.name, entity=self.entity.to_proto()
            )
        )


@dataclass(unsafe_hash=True)
class StreamingFeature(BaseFeature):
    entity: Entity

    def to_proto(self):
        return ProtoFeature(
            streaming_feature=ProtoStreamingFeature(
                name=self.name, entity=self.entity.to_proto()
            )
        )


@dataclass(unsafe_hash=True)
class StreamingFeatureV1(BaseFeature):
    entity: Optional[Entity] = None

    def to_proto(self):
        return ProtoFeature(
            streaming_feature_v1=ProtoStreamingFeatureV1(
                name=self.name, entity=self.entity.to_proto()
            )
        )


@dataclass(unsafe_hash=True)
class StreamingAggregationFeature(BaseFeature):
    entity: Optional[Entity] = None

    def to_proto(self):
        return ProtoFeature(
            streaming_aggregation_feature=ProtoStreamingAggregationFeature(
                name=self.name, entity=self.entity.to_proto()
            )
        )


@dataclass()
class FeatureSetInfo:
    entity_spec: ProtoEntitySpec
    feature_set_type: ProtoFeatureSetType
    feature_version: int


def get_feature_set_info(
    feature_manager_client: FeatureRegistryClient, feature_set_name: str
) -> FeatureSetInfo:
    """
    Get the entities by the feature set name and feature type
    Args:
        feature_manager_client: feature manager client for the grpc request
        feature_set_name: the required feature set name
    Returns: tuple of entity spec, type and feature version

    """
    feature_set_response = feature_manager_client.get_feature_set_by_name(
        feature_set_name
    )
    if not feature_set_response:
        raise QwakException(f"Featureset '{feature_set_name}' does not exist")

    featureset_type: ProtoFeatureSetType = get_feature_type(
        feature_set_response.feature_set.feature_set_definition.feature_set_spec.feature_set_type
    )
    featureset_version = _get_featureset_version(featureset_type)

    return FeatureSetInfo(
        entity_spec=feature_set_response.feature_set.feature_set_definition.feature_set_spec.entity.entity_spec,
        feature_set_type=featureset_type,
        feature_version=featureset_version,
    )


def get_feature_type(feature_set_type: ProtoFeatureSetType):
    return getattr(feature_set_type, feature_set_type.WhichOneof("set_type"))


def _get_featureset_version(feature_set_type: ProtoFeatureSetType) -> int:
    """
    Get Featureset version. Return 0 if none version is set
    """
    if hasattr(feature_set_type, "qwak_internal_protocol_version"):
        return feature_set_type.qwak_internal_protocol_version
    return 0


def get_typed_feature(
    feature: FeatureStoreInput,
    feature_type: ProtoFeatureSetType,
    featureset_version: int,
) -> BaseFeature:
    """
    convert InputFeature to the relevant type
    Args:
        feature: Input feature to cast to the correct type
        feature_type: the feature type as it registered
        featureset_version: the version of the featureset

    Return:
        BaseFeature with the correct type
    """
    if isinstance(feature_type, ProtoBatchFeatureSet):
        return BatchFeature(name=feature.name, entity=feature.entity)
    elif isinstance(feature_type, ProtoBatchFeatureSetV1) and featureset_version == 0:
        return BatchFeature(name=feature.name, entity=feature.entity)
    elif isinstance(feature_type, ProtoBatchFeatureSetV1) and featureset_version != 0:
        return BatchFeatureV1(name=feature.name, entity=feature.entity)
    elif (
        isinstance(feature_type, ProtoStreamingFeatureSetV1) and featureset_version == 0
    ):
        return StreamingFeature(name=feature.name, entity=feature.entity)
    elif (
        isinstance(feature_type, ProtoStreamingFeatureSetV1) and featureset_version != 0
    ):
        return StreamingFeatureV1(name=feature.name, entity=feature.entity)
    elif isinstance(feature_type, ProtoStreamingAggregationFeatureSet):
        return StreamingAggregationFeature(name=feature.name, entity=feature.entity)
    else:
        raise ValueError(
            f"Feature set type {feature_type} with protocol version {featureset_version} is not supported for extraction"
        )


def get_env_to_featuresets_mapping(
    feature_manager_client: FeatureRegistryClient,
) -> Dict[str, Dict[str, FeatureSetInfo]]:
    """
    Get mapping of the account's environment name to feature sets and converts each FeatureSet to FeatureSetInfo
    :param feature_manager_client:
    :return: Dict[str, List[FeatureSetInfo]] where the key is the environment name and the value is a list of FeatureSetInfo
    """
    env_to_featuresets_mapping = feature_manager_client.get_env_to_featuresets_mapping()

    # Convert the mapping to a dictionary of type Dict[str, List[FeatureSetInfo]]
    result = {}
    for (
        env_name,
        feature_sets_infos,
    ) in env_to_featuresets_mapping.env_to_feature_set_mapping.items():
        env_name = env_name.lower()
        result[env_name] = {}
        for feature_set in feature_sets_infos.feature_sets:
            featureset_type: ProtoFeatureSetType = get_feature_type(
                feature_set.feature_set_definition.feature_set_spec.feature_set_type
            )
            featureset_version = _get_featureset_version(featureset_type)

            feature_set_info = FeatureSetInfo(
                entity_spec=feature_set.feature_set_definition.feature_set_spec.entity.entity_spec,
                feature_set_type=featureset_type,
                feature_version=featureset_version,
            )

            feature_set_name = (
                feature_set.feature_set_definition.feature_set_spec.name.lower()
            )
            result[env_name][feature_set_name] = feature_set_info
    return result


def get_entity_type(value_type: ProtoEntitySpec.ValueType):
    """
    Normalize entity by the enum
    """

    if value_type == ProtoEntitySpec.ValueType.STRING:
        return str
    elif value_type == ProtoEntitySpec.ValueType.INT:
        return int


def generate_key_unique_name(featureset_name: str):
    """
    Generate a name for the feature set key to be registered or treated as a Qwak entity based on its' featuresets'
    name
    """
    return f"{_INTERNAL_KEY_PREFIX}_{featureset_name}_{round(time.time() * 1000)}"


def get_batch_source_for_featureset(
    batch_ds_name: str, feature_registry: FeatureRegistryClient
):
    batch_ds = feature_registry.get_data_source_by_name(batch_ds_name)
    if not batch_ds:
        raise QwakException(
            f"Trying to register a feature set with a non registered data source -: {batch_ds_name}"
        )

    ds_spec = batch_ds.data_source.data_source_definition.data_source_spec

    if ds_spec.WhichOneof("type") != "batch_source":
        raise ValueError(
            f"Can only register streaming backfill/batch featuresets with batch sources. "
            f"Source {batch_ds_name} is of type {ds_spec.WhichOneof('type')}"
        )

    return ds_spec.batch_source



import inspect
import os
from dataclasses import dataclass, field
from pathlib import Path
from typing import Callable, Dict, Optional, Type

from _qwak_proto.qwak.feature_store.sources.data_source_pb2 import (
    DataSourceSpec as ProtoDataSourceSpec,
)
from _qwak_proto.qwak.feature_store.sources.streaming_pb2 import (
    KafkaSourceV1 as ProtoKafkaSourceV1,
    StreamingSource as ProtoStreamingSource,
)
from qwak.exceptions import QwakException
from qwak.feature_store._common.artifact_utils import ArtifactSpec
from qwak.feature_store.data_sources.streaming._streaming import BaseStreamingSource
from qwak.feature_store.data_sources.streaming.kafka.authentication import (
    BaseAuthentication,
    SslAuthentication,
)
from qwak.feature_store.data_sources.streaming.kafka.deserialization import Deserializer


@dataclass
class KafkaSource(BaseStreamingSource):
    bootstrap_servers: str

    # Deserialization
    deserialization: Deserializer

    # secret configs, the value is resolved to the secret,
    # s.t. (key, value) -> (key, get_secret(value))
    # not all configs will be respected, this is a best-effort
    secret_configs: Dict[str, str] = field(default_factory=lambda: {})

    # passthrough configs - not all configs will be respected,
    # this is a best-effort
    passthrough_configs: Dict[str, str] = field(default_factory=lambda: {})

    # the following 3 are pairwise mutually exclusive
    assign: Optional[str] = None
    subscribe: Optional[str] = None
    subscribe_pattern: Optional[str] = None

    authentication_method: Optional[BaseAuthentication] = field(
        default_factory=lambda: SslAuthentication()
    )

    def __post_init__(self):
        self._validate()

    def _validate(self):
        num_defined = len(
            [
                _
                for _ in [self.assign, self.subscribe, self.subscribe_pattern]
                if _ is not None
            ]
        )
        if num_defined != 1:
            raise QwakException(
                "Exactly one of (assign, subscribe, subscribe_pattern) must be defined!"
            )

    def _get_artifacts(self) -> Optional["ArtifactSpec"]:
        deserializer_function: Optional[Callable] = self.deserialization._get_function()

        if deserializer_function:
            return ArtifactSpec(
                artifact_name=self.name,
                root_module_path=Path(
                    os.path.abspath(inspect.getfile(deserializer_function))
                ).parent,
                artifact_object=self.deserialization,
                callables=[deserializer_function],
                suffix="resource.zip",
            )

    def _to_proto(self, artifact_url: Optional[str] = None) -> ProtoDataSourceSpec:
        return ProtoDataSourceSpec(
            stream_source=ProtoStreamingSource(
                name=self.name,
                description=self.description,
                kafkaSourceV1=ProtoKafkaSourceV1(
                    bootstrap_servers=self.bootstrap_servers,
                    assign=self.assign,
                    subscribe=self.subscribe,
                    subscribe_pattern=self.subscribe_pattern,
                    secret_configs=self.secret_configs,
                    passthrough_configs=self.passthrough_configs,
                    authentication_method=self.authentication_method._to_proto(),
                    deserialization=self.deserialization._to_proto(
                        artifact_path=artifact_url
                    ),
                ),
            )
        )

    @classmethod
    def _from_proto(cls, proto) -> Type["KafkaSource"]:
        kafka = proto.kafkaSourceV1
        topic_configuration_key = kafka.WhichOneof("topic_configuration")
        oneof_args = {topic_configuration_key: getattr(kafka, topic_configuration_key)}
        return cls(
            name=proto.name,
            description=proto.description,
            bootstrap_servers=kafka.bootstrap_servers,
            secret_configs=kafka.secret_configs,
            passthrough_configs=kafka.passthrough_configs,
            deserialization=Deserializer._from_proto(
                proto_deserializer=kafka.deserialization
            ),
            authentication_method=BaseAuthentication._from_proto(
                proto_authentication_method=kafka.authentication_method
            ),
            **oneof_args,
        )



import re
from abc import ABC, abstractmethod
from dataclasses import dataclass, field
from typing import Dict, List, Tuple, Union

from qwak.llmops.prompt.chat.message import (
    AIMessage,
    BaseMessage,
    HumanMessage,
    SystemMessage,
)
from qwak.llmops.prompt.chat.value import ChatPromptValue
from qwak.llmops.prompt.template import BasePromptTemplate, StringPromptTemplate


@dataclass
class BaseMessagePromptTemplate(BasePromptTemplate):
    @abstractmethod
    def render(self, variables: Dict[str, any]) -> BaseMessage:
        pass


@dataclass
class BaseStringMessagePromptTemplate(BaseMessagePromptTemplate, ABC):
    template: StringPromptTemplate = field(init=False)
    role_name: str = field(init=False)

    def __init__(self, template: str):
        self.template = StringPromptTemplate(template=template)


class AIMessagePromptTemplate(BaseStringMessagePromptTemplate):
    role_name: str = "ai"

    def render(self, variables: Dict[str, any]) -> BaseMessage:
        return AIMessage(content=self.template.render(variables=variables).to_string())


class HumanMessagePromptTemplate(BaseStringMessagePromptTemplate):
    role_name: str = "human"

    def render(self, variables: Dict[str, any]) -> BaseMessage:
        return HumanMessage(
            content=self.template.render(variables=variables).to_string()
        )


class SystemMessagePromptTemplate(BaseStringMessagePromptTemplate):
    role_name: str = "system"

    def render(self, variables: Dict[str, any]) -> BaseMessage:
        return SystemMessage(
            content=self.template.render(variables=variables).to_string()
        )


@dataclass
class ChatPromptTemplate(BasePromptTemplate):
    messages: List[Union[BaseMessage, BaseStringMessagePromptTemplate]]

    def render(self, variables: Dict[str, any]) -> ChatPromptValue:
        resulting_messages: List[BaseMessage] = list()

        for message in self.messages:
            if isinstance(message, BaseMessage):
                resulting_messages.append(message)
            elif isinstance(message, BaseStringMessagePromptTemplate):
                resulting_messages.append(message.render(variables=variables))
            else:
                raise ValueError(
                    f"Got unsupported message type: {repr(message)}. \n"
                    "Supported messages are: "
                    "AIMessagePromptTemplate, HumanMessagePromptTemplate, SystemMessagePromptTemplate, \n"
                    "AIMessage, HumanMessage, SystemMessage."
                )

        return ChatPromptValue(messages=resulting_messages)

    def to_messages(self) -> List[Tuple[str, str]]:
        """
        Useful for integration with other libraries such as Langchain.

        ```
        ChatPromptTemplate(
                messages=[
                    SystemMessage("you are an assistant"),
                    HumanMessagePromptTemplate("{{question}}")
                ]
            ).to_messages()

        resulting in:

        [("system", "you are an assistant"),
        ("human", "{question}")]
        ```

        """

        def strip_curly(string: str) -> str:
            return re.sub(r"\{\{\s*([\w\s]+)\s*\}\}", repl=r"{\g<1>}", string=string)

        if not self.messages:
            return []

        result: List[Tuple[str, str]] = []

        for msg in self.messages:
            if isinstance(msg, BaseMessage):
                result.append((msg.role_name, msg.content))
            elif isinstance(msg, BaseStringMessagePromptTemplate):
                result.append((msg.role_name, strip_curly(msg.template.template)))

        return result



from _qwak_proto.qwak.instance_template.instance_template_pb2 import (
    InstanceTemplateSpec,
    InstanceType,
)
from _qwak_proto.qwak.instance_template.instance_template_service_pb2 import (
    GetInstanceTemplateRequest,
    GetInstanceTemplateResponse,
    ListInstanceTemplatesRequest,
    ListInstanceTemplatesResponse,
)
from _qwak_proto.qwak.instance_template.instance_template_service_pb2_grpc import (
    InstanceTemplateManagementServiceServicer,
)
from _qwak_proto.qwak.user_application.common.v0.resources_pb2 import (
    CpuResources,
    GpuResources,
    GpuType,
    MemoryUnit,
    NodeOptimizationType,
    PodComputeResources,
)
from qwak.exceptions import QwakException
from qwak_services_mock.mocks.utils.exception_handlers import raise_internal_grpc_error


def _create_instance_template_cpu(
    instance_id: str,
    name: str,
    order: int,
    cpu: float,
    memory_amount: int,
    aws_supported: bool = True,
    gcp_supported: bool = True,
) -> InstanceTemplateSpec:
    return InstanceTemplateSpec(
        id=instance_id,
        display_name=name,
        order=order,
        enabled=True,
        aws_supported=aws_supported,
        gcp_supported=gcp_supported,
        pod_compute_resources=PodComputeResources(
            optimization_type=NodeOptimizationType.NODE_OPTIMIZATION_NONE,
            cpu_resources=CpuResources(
                cpu=cpu,
                memory_amount=memory_amount,
                memory_units=MemoryUnit.GIB,
            ),
        ),
        instance_type=InstanceType.INSTANCE_TYPE_CPU,
    )


def _create_instance_template_gpu(
    instance_id: str,
    name: str,
    order: int,
    gpu: int,
    gpu_type: GpuType,
    cpu: float,
    memory_amount: int,
    aws_supported: bool = True,
    gcp_supported: bool = True,
) -> InstanceTemplateSpec:
    return InstanceTemplateSpec(
        id=instance_id,
        display_name=name,
        order=order,
        enabled=True,
        aws_supported=aws_supported,
        gcp_supported=gcp_supported,
        pod_compute_resources=PodComputeResources(
            optimization_type=NodeOptimizationType.NODE_OPTIMIZATION_NONE,
            gpu_resources=GpuResources(
                gpu_amount=gpu,
                gpu_type=gpu_type,
            ),
            cpu_resources=CpuResources(
                cpu=cpu,
                memory_amount=memory_amount,
                memory_units=MemoryUnit.GIB,
            ),
        ),
        instance_type=InstanceType.INSTANCE_TYPE_GPU,
    )


class InstanceTemplateManagementServiceMock(InstanceTemplateManagementServiceServicer):
    INSTANCES = {
        "small": _create_instance_template_cpu("small", "Small", 1, 1, 8),
        "medium": _create_instance_template_cpu("medium", "Medium", 2, 2, 16),
        "large": _create_instance_template_cpu("large", "Large", 3, 4, 32),
        "xlarge": _create_instance_template_cpu("xlarge", "XLarge", 4, 8, 64),
        "g5_xlarge": _create_instance_template_gpu(
            "g5_xlarge", "G5 XLarge", 1, 1, GpuType.NVIDIA_A10G, 1, 8
        ),
        "g5_2xlarge": _create_instance_template_gpu(
            "g5_2xlarge", "G5 2XLarge", 2, 2, GpuType.NVIDIA_A10G, 2, 16
        ),
        "g5_4xlarge": _create_instance_template_gpu(
            "g5_4xlarge", "G5 4XLarge", 2, 4, GpuType.NVIDIA_A10G, 4, 32
        ),
        "aws_only": _create_instance_template_gpu(
            "aws_only",
            "Aws Only",
            2,
            4,
            GpuType.NVIDIA_A10G,
            4,
            32,
            aws_supported=True,
            gcp_supported=False,
        ),
        "gcp_only": _create_instance_template_gpu(
            "gcp_only",
            "GCP Only",
            2,
            4,
            GpuType.NVIDIA_A10G,
            4,
            32,
            aws_supported=False,
            gcp_supported=True,
        ),
    }

    def ListInstanceTemplates(
        self, request: ListInstanceTemplatesRequest, context
    ) -> ListInstanceTemplatesResponse:
        try:
            return ListInstanceTemplatesResponse(
                instance_template_list=list(self.INSTANCES.values())
            )
        except Exception as e:
            raise_internal_grpc_error(context, e)

    def GetInstanceTemplate(
        self, request: GetInstanceTemplateRequest, context
    ) -> GetInstanceTemplateResponse:
        try:
            if request.id in self.INSTANCES:
                return GetInstanceTemplateResponse(
                    instance_template=self.INSTANCES[request.id]
                )
            else:
                raise QwakException("Instance template not found")
        except Exception as e:
            raise_internal_grpc_error(context, e)



from typing import Any

from qwak.exceptions import QwakException, QwakHTTPException
from qwak.model.adapters import (
    AutodetectOutputAdapter,
    DataFrameOutputAdapter,
    DefaultOutputAdapter,
    JsonOutputAdapter,
    NumpyOutputAdapter,
    ProtoOutputAdapter,
    TfTensorOutputAdapter,
)
from qwak.model.base import QwakModel

from .output_adapters.dataframe_output import DataFrameOutput
from .output_adapters.default_output import DefaultOutput
from .output_adapters.json_output import JsonOutput
from .output_adapters.tf_tensor_output import TfTensorOutput


def get_output_adapter(
    model: QwakModel,
):
    adapter = getattr(model.predict, "_output_adapter", "")

    class QwakOutput(JsonOutput):
        def __init__(self, **kwargs):
            super().__init__(**kwargs)

        def pack_user_func_return_value(self, return_result: Any) -> Any:
            try:
                return adapter.pack_user_func_return_value(return_result)
            except Exception as e:
                raise QwakHTTPException(
                    message=f"Failed to serialize model output. Error is: {e}. For more "
                    f"information please check model logs.",
                    status_code=500,
                )

    class QwakOutputWithDefaultFallbackOutput(QwakOutput):
        def __init__(self, **kwargs):
            super().__init__(**kwargs)

        def pack_user_func_return_value(self, return_result: Any) -> Any:
            from google.protobuf.message import Message

            first_result = (
                return_result[0]
                if type(return_result) == list and len(return_result) > 0
                else return_result
            )
            if issubclass(type(first_result), Message):
                response = QwakOutput().pack_user_func_return_value(return_result)
                return response
            else:
                return DefaultOutput().pack_user_func_return_value(return_result)

    mapping = {
        JsonOutputAdapter: JsonOutput,
        DataFrameOutputAdapter: DataFrameOutput,
        DefaultOutputAdapter: DefaultOutput,
        AutodetectOutputAdapter: QwakOutputWithDefaultFallbackOutput,
        ProtoOutputAdapter: QwakOutput,
        NumpyOutputAdapter: QwakOutput,
        TfTensorOutputAdapter: TfTensorOutput,
    }
    for qwak_impl, runtime_impl in mapping.items():
        if isinstance(adapter, qwak_impl):
            if isinstance(adapter, DataFrameOutputAdapter) and hasattr(
                adapter, "output_orient"
            ):
                return runtime_impl(output_orient=adapter.output_orient)
            return runtime_impl()

    raise QwakException("No Adapter selected for output")



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import qwak.feature_store.serving.metadata_pb2
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class GetFeaturesetMemoryUsageByIdRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FEATURESET_ID_FIELD_NUMBER: builtins.int
    featureset_id: builtins.str
    def __init__(
        self,
        *,
        featureset_id: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["featureset_id", b"featureset_id"]) -> None: ...

global___GetFeaturesetMemoryUsageByIdRequest = GetFeaturesetMemoryUsageByIdRequest

class GetFeaturesetMemoryUsageByIdResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    USAGE_FIELD_NUMBER: builtins.int
    @property
    def usage(self) -> qwak.feature_store.serving.metadata_pb2.FeaturesetMemoryUsageStats: ...
    def __init__(
        self,
        *,
        usage: qwak.feature_store.serving.metadata_pb2.FeaturesetMemoryUsageStats | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["usage", b"usage"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["usage", b"usage"]) -> None: ...

global___GetFeaturesetMemoryUsageByIdResponse = GetFeaturesetMemoryUsageByIdResponse

class GetFeaturesetMemoryUsageByEnvironmentRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___GetFeaturesetMemoryUsageByEnvironmentRequest = GetFeaturesetMemoryUsageByEnvironmentRequest

class GetFeaturesetMemoryUsageByEnvironmentResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    USAGE_STATS_FIELD_NUMBER: builtins.int
    @property
    def usage_stats(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[qwak.feature_store.serving.metadata_pb2.FeaturesetMemoryUsageStats]: ...
    def __init__(
        self,
        *,
        usage_stats: collections.abc.Iterable[qwak.feature_store.serving.metadata_pb2.FeaturesetMemoryUsageStats] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["usage_stats", b"usage_stats"]) -> None: ...

global___GetFeaturesetMemoryUsageByEnvironmentResponse = GetFeaturesetMemoryUsageByEnvironmentResponse



import re
from abc import ABC, abstractmethod
from dataclasses import dataclass
from typing import TYPE_CHECKING, List, Optional, Tuple

from _qwak_proto.qwak.feature_store.entities.entity_pb2 import (
    EntityDefinition,
    EntitySpec,
)
from _qwak_proto.qwak.feature_store.features.feature_set_pb2 import FeatureSetSpec
from qwak.feature_store._common.feature_set_utils import generate_key_unique_name
from qwak.feature_store.entities.entity import Entity
from qwak.feature_store.validations.validation_response import SuccessValidationResponse

if TYPE_CHECKING:
    try:
        import pandas as pd
    except ImportError:
        pass

from qwak.clients.feature_store import FeatureRegistryClient
from qwak.exceptions import QwakException
from qwak.feature_store.validations.validation_options import (
    FeatureSetValidationOptions,
)


@dataclass
class BaseFeatureSet(ABC):
    name: str
    data_sources: List[str]
    entity: str = str()
    key: str = str()
    __instance_module_path__: Optional[str] = None

    def _validate(self):
        self._validate_feature_set_name()

    def _validate_feature_set_name(self):
        """
        Verifies that the name follows these rules (rfc 1123):
        * A name can start or end with a letter or a number
        * A name MUST NOT start or end with a '-' (dash)
        * A name MUST NOT consist of all numeric values
        * A name can be up to 63 characters
        """
        regex = re.compile(r"^(?![_-])\D[a-zA-Z0-9_-]{0,61}[a-zA-Z0-9](?![_-])$")
        if not regex.match(self.name):
            raise QwakException(
                f"feature set name: {self.name} is invalid, name must follow these conditions:"
                f"can start or end with a letter or a number"
                f"name MUST NOT start or end with a '-' (dash)"
                f"name MUST NOT consist of all numeric values"
            )

    def _get_entity_definition(
        self, feature_registry: FeatureRegistryClient
    ) -> EntityDefinition:
        # fs has a key def but not an entity def, return an entity that is valid but not registered
        # This is done mainly for the validation flow/get sample, to avoid registering temporary entities for keys
        if self.key and not self.entity:
            new_key_name: str = generate_key_unique_name(self.name)

            new_key_spec: EntitySpec = Entity(
                name=new_key_name,
                description=f"Key of feature set {self.name}",
                key=self.key,
            )._to_proto()

            # return new entity def with no id
            entity_definition: EntityDefinition = EntityDefinition(
                entity_spec=new_key_spec
            )
        else:
            feature_set_entity = feature_registry.get_entity_by_name(self.entity)

            if not feature_set_entity:
                raise QwakException(
                    f"Trying to register a feature set with a non existing entity or bad key configuration-: {self.entity}"
                )

            entity_definition: EntityDefinition = (
                feature_set_entity.entity.entity_definition
            )

        return entity_definition

    @staticmethod
    @abstractmethod
    def _from_proto(cls, proto: FeatureSetSpec):
        pass

    @abstractmethod
    def _to_proto(
        self,
        git_commit,
        features,
        feature_registry,
        artifact_url: Optional[str] = None,
        **kwargs,
    ) -> Tuple[FeatureSetSpec, Optional[str]]:
        pass

    def get_sample(
        self,
        number_of_rows: int = 10,
        validation_options: Optional[FeatureSetValidationOptions] = None,
    ) -> "pd.DataFrame":
        """
        Fetches a sample of the Feature set transformation by loading requested sample of data from the data source
        and executing the transformation on that data.

        :param number_of_rows: number of rows requests
        :param validation_options: validation options
        :returns Sample Pandas Dataframe

        Example:

        ... code-block:: python
            @batch.feature_set(entity="users", data_sources=["snowflake_users_table"])
            @batch.backfill(start_date=datetime(2022, 1, 1))
            def user_features():
                return SparkSqlTransformation("SELECT user_id, age FROM data_source")

            sample_features = user_features.get_sample()
            print(sample_feature)
            #	    user_id	         timestamp	        user_features.age
            # 0	      1	        2021-01-02 17:00:00	              23
            # 1	      1	        2021-01-01 12:00:00	              51
            # 2	      2	        2021-01-02 12:00:00	              66
            # 3	      2	        2021-01-01 18:00:00	              34
        """
        from qwak.feature_store.validations.validator import FeaturesOperatorValidator

        v = FeaturesOperatorValidator()

        response, _ = v.validate_featureset(
            featureset=self,
            sample_size=number_of_rows,
            validation_options=validation_options,
        )

        if isinstance(response, SuccessValidationResponse):
            return response.sample
        else:
            raise QwakException(f"Sampling failed: \n{response}")  # noqa: E231



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/feature_store/serving/v1/value.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n)qwak/feature_store/serving/v1/value.proto\x12$com.qwak.ai.feature.store.serving.v1\x1a\x1fgoogle/protobuf/timestamp.proto\"\xad\x02\n\x07ValueV1\x12\x0e\n\x06is_set\x18\x01 \x01(\x08\x12\x13\n\tbytes_val\x18\x02 \x01(\x0cH\x00\x12\x14\n\nstring_val\x18\x03 \x01(\tH\x00\x12\x13\n\tint32_val\x18\x04 \x01(\x05H\x00\x12\x13\n\tint64_val\x18\x05 \x01(\x03H\x00\x12\x14\n\ndouble_val\x18\x06 \x01(\x01H\x00\x12\x13\n\tfloat_val\x18\x07 \x01(\x02H\x00\x12\x12\n\x08\x62ool_val\x18\x08 \x01(\x08H\x00\x12\x33\n\rtimestamp_val\x18\t \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x00\x12\x42\n\tarray_val\x18\n \x01(\x0b\x32-.com.qwak.ai.feature.store.serving.v1.ArrayV1H\x00\x42\x05\n\x03val\"K\n\x07\x41rrayV1\x12@\n\tarray_val\x18\x01 \x03(\x0b\x32-.com.qwak.ai.feature.store.serving.v1.ValueV1B&\n$com.qwak.ai.feature.store.serving.v1b\x06proto3')



_VALUEV1 = DESCRIPTOR.message_types_by_name['ValueV1']
_ARRAYV1 = DESCRIPTOR.message_types_by_name['ArrayV1']
ValueV1 = _reflection.GeneratedProtocolMessageType('ValueV1', (_message.Message,), {
  'DESCRIPTOR' : _VALUEV1,
  '__module__' : 'qwak.feature_store.serving.v1.value_pb2'
  # @@protoc_insertion_point(class_scope:com.qwak.ai.feature.store.serving.v1.ValueV1)
  })
_sym_db.RegisterMessage(ValueV1)

ArrayV1 = _reflection.GeneratedProtocolMessageType('ArrayV1', (_message.Message,), {
  'DESCRIPTOR' : _ARRAYV1,
  '__module__' : 'qwak.feature_store.serving.v1.value_pb2'
  # @@protoc_insertion_point(class_scope:com.qwak.ai.feature.store.serving.v1.ArrayV1)
  })
_sym_db.RegisterMessage(ArrayV1)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n$com.qwak.ai.feature.store.serving.v1'
  _VALUEV1._serialized_start=117
  _VALUEV1._serialized_end=418
  _ARRAYV1._serialized_start=420
  _ARRAYV1._serialized_end=495
# @@protoc_insertion_point(module_scope)



from dataclasses import dataclass, field
from datetime import datetime
from enum import Enum

from _qwak_proto.qwak.batch_job.v1.batch_job_service_pb2 import (
    BatchJobDetails as BatchJobDetailsProto,
)


class ExecutionStatus(Enum):
    UNDEFINED_BATCH_JOB_STATUS = 0
    BATCH_JOB_COMMITTED_STATUS = 1
    BATCH_JOB_PENDING_STATUS = 2
    BATCH_JOB_RUNNING_STATUS = 3
    BATCH_JOB_FINISHED_STATUS = 4
    BATCH_JOB_FAILED_STATUS = 5
    BATCH_JOB_CANCELLED_STATUS = 6
    BATCH_JOB_TIMEOUT_STATUS = 7


@dataclass
class Execution:
    environment_id: str = field(default=None)
    build_id: str = field(default=None)
    execution_id: str = field(default=None)
    execution_status: ExecutionStatus = field(
        default=ExecutionStatus.UNDEFINED_BATCH_JOB_STATUS
    )
    start_time: datetime = field(default=datetime.now())
    end_time: datetime = field(default=datetime.now())
    failure_message: str = field(default=None)

    @staticmethod
    def from_proto(batch_job_details_proto: BatchJobDetailsProto):
        return Execution(
            environment_id=batch_job_details_proto.environment_id,
            build_id=batch_job_details_proto.build_id,
            execution_id=batch_job_details_proto.job_id,
            execution_status=ExecutionStatus(batch_job_details_proto.job_status),
            start_time=datetime.fromtimestamp(
                batch_job_details_proto.start_time.seconds
                + batch_job_details_proto.start_time.nanos / 1e9
            ),
            end_time=datetime.fromtimestamp(
                batch_job_details_proto.end_time.seconds
                + batch_job_details_proto.end_time.nanos / 1e9
            ),
            failure_message=batch_job_details_proto.failure_message,
        )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/execution/v1/execution.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from _qwak_proto.qwak.execution.v1 import backfill_pb2 as qwak_dot_execution_dot_v1_dot_backfill__pb2
from _qwak_proto.qwak.execution.v1 import batch_pb2 as qwak_dot_execution_dot_v1_dot_batch__pb2
from _qwak_proto.qwak.execution.v1 import streaming_pb2 as qwak_dot_execution_dot_v1_dot_streaming__pb2
from _qwak_proto.qwak.execution.v1 import streaming_aggregation_pb2 as qwak_dot_execution_dot_v1_dot_streaming__aggregation__pb2
from _qwak_proto.qwak.execution.v1 import deletion_pb2 as qwak_dot_execution_dot_v1_dot_deletion__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n!qwak/execution/v1/execution.proto\x12\x1fqwak.feature.store.execution.v1\x1a\x1fgoogle/protobuf/timestamp.proto\x1a qwak/execution/v1/backfill.proto\x1a\x1dqwak/execution/v1/batch.proto\x1a!qwak/execution/v1/streaming.proto\x1a-qwak/execution/v1/streaming_aggregation.proto\x1a qwak/execution/v1/deletion.proto\"\x8e\n\n\x0e\x45xecutionEntry\x12\x15\n\rfeatureset_id\x18\x01 \x01(\t\x12\x14\n\x0c\x65xecution_id\x18\x02 \x01(\t\x12J\n\x10\x65xecution_status\x18\x03 \x01(\x0e\x32\x30.qwak.feature.store.execution.v1.ExecutionStatus\x12\x30\n\x0csubmitted_at\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x45\n\x0csubmitted_by\x18\x05 \x01(\x0b\x32/.qwak.feature.store.execution.v1.InvokerDetails\x12\x30\n\x0ctriggered_at\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12,\n\x08\x65nded_at\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x46\n\rbackfill_spec\x18\x08 \x01(\x0b\x32-.qwak.feature.store.execution.v1.BackfillSpecH\x00\x12J\n\x0f\x62\x61tch_ingestion\x18\t \x01(\x0b\x32/.qwak.feature.store.execution.v1.BatchIngestionH\x00\x12R\n\x13\x66\x65\x61tureset_deletion\x18\n \x01(\x0b\x32\x33.qwak.feature.store.execution.v1.FeaturesetDeletionH\x00\x12_\n\x1aonline_streaming_ingestion\x18\x0b \x01(\x0b\x32\x39.qwak.feature.store.execution.v1.OnlineStreamingIngestionH\x00\x12\x61\n\x1boffline_streaming_ingestion\x18\x0c \x01(\x0b\x32:.qwak.feature.store.execution.v1.OfflineStreamingIngestionH\x00\x12{\n)row_level_streaming_aggregation_ingestion\x18\r \x01(\x0b\x32\x46.qwak.feature.store.execution.v1.RowLevelStreamingAggregationIngestionH\x00\x12~\n*compaction_streaming_aggregation_ingestion\x18\x0e \x01(\x0b\x32H.qwak.feature.store.execution.v1.CompactionStreamingAggregationIngestionH\x00\x12z\n(streaming_aggregation_backfill_ingestion\x18\x0f \x01(\x0b\x32\x46.qwak.feature.store.execution.v1.StreamingAggregationBackfillIngestionH\x00\x12Y\n\x12\x64\x65ployment_details\x18\x10 \x01(\x0b\x32;.qwak.feature.store.execution.v1.ExecutionDeploymentDetailsH\x01\x42\x10\n\x0e\x65xecution_typeB\x18\n\x16\x64\x65ployment_details_opt\"Y\n\x1a\x45xecutionDeploymentDetails\x12\x1c\n\x12\x66\x65\x61tureset_version\x18\x01 \x01(\x05H\x00\x42\x1d\n\x1b\x64\x65ployed_featureset_version\"\xbc\x01\n\x0eInvokerDetails\x12L\n\x10\x65xternal_invoker\x18\x01 \x01(\x0b\x32\x30.qwak.feature.store.execution.v1.ExternalInvokerH\x00\x12L\n\x10internal_invoker\x18\x02 \x01(\x0b\x32\x30.qwak.feature.store.execution.v1.InternalInvokerH\x00\x42\x0e\n\x0cinvoker_type\"(\n\x0f\x45xternalInvoker\x12\x15\n\rinvoker_email\x18\x01 \x01(\t\"\x11\n\x0fInternalInvoker*\xcb\x02\n\x0f\x45xecutionStatus\x12\x1c\n\x18\x45XECUTION_STATUS_INVALID\x10\x00\x12\x1b\n\x17\x45XECUTION_STATUS_QUEUED\x10\x01\x12\x1c\n\x18\x45XECUTION_STATUS_PENDING\x10\x02\x12\x1c\n\x18\x45XECUTION_STATUS_RUNNING\x10\x03\x12\x1e\n\x1a\x45XECUTION_STATUS_COMPLETED\x10\x04\x12\x1b\n\x17\x45XECUTION_STATUS_FAILED\x10\x05\x12\x1e\n\x1a\x45XECUTION_STATUS_CANCELLED\x10\x06\x12\x1e\n\x1a\x45XECUTION_STATUS_SUBMITTED\x10\x07\x12&\n\"EXECUTION_STATUS_FAILED_SUBMISSION\x10\x08\x12\x1c\n\x18\x45XECUTION_STATUS_SKIPPED\x10\tBN\n%com.qwak.ai.features.execution.api.v1P\x01Z#qwak/execution_management;executionb\x06proto3')

_EXECUTIONSTATUS = DESCRIPTOR.enum_types_by_name['ExecutionStatus']
ExecutionStatus = enum_type_wrapper.EnumTypeWrapper(_EXECUTIONSTATUS)
EXECUTION_STATUS_INVALID = 0
EXECUTION_STATUS_QUEUED = 1
EXECUTION_STATUS_PENDING = 2
EXECUTION_STATUS_RUNNING = 3
EXECUTION_STATUS_COMPLETED = 4
EXECUTION_STATUS_FAILED = 5
EXECUTION_STATUS_CANCELLED = 6
EXECUTION_STATUS_SUBMITTED = 7
EXECUTION_STATUS_FAILED_SUBMISSION = 8
EXECUTION_STATUS_SKIPPED = 9


_EXECUTIONENTRY = DESCRIPTOR.message_types_by_name['ExecutionEntry']
_EXECUTIONDEPLOYMENTDETAILS = DESCRIPTOR.message_types_by_name['ExecutionDeploymentDetails']
_INVOKERDETAILS = DESCRIPTOR.message_types_by_name['InvokerDetails']
_EXTERNALINVOKER = DESCRIPTOR.message_types_by_name['ExternalInvoker']
_INTERNALINVOKER = DESCRIPTOR.message_types_by_name['InternalInvoker']
ExecutionEntry = _reflection.GeneratedProtocolMessageType('ExecutionEntry', (_message.Message,), {
  'DESCRIPTOR' : _EXECUTIONENTRY,
  '__module__' : 'qwak.execution.v1.execution_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.execution.v1.ExecutionEntry)
  })
_sym_db.RegisterMessage(ExecutionEntry)

ExecutionDeploymentDetails = _reflection.GeneratedProtocolMessageType('ExecutionDeploymentDetails', (_message.Message,), {
  'DESCRIPTOR' : _EXECUTIONDEPLOYMENTDETAILS,
  '__module__' : 'qwak.execution.v1.execution_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.execution.v1.ExecutionDeploymentDetails)
  })
_sym_db.RegisterMessage(ExecutionDeploymentDetails)

InvokerDetails = _reflection.GeneratedProtocolMessageType('InvokerDetails', (_message.Message,), {
  'DESCRIPTOR' : _INVOKERDETAILS,
  '__module__' : 'qwak.execution.v1.execution_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.execution.v1.InvokerDetails)
  })
_sym_db.RegisterMessage(InvokerDetails)

ExternalInvoker = _reflection.GeneratedProtocolMessageType('ExternalInvoker', (_message.Message,), {
  'DESCRIPTOR' : _EXTERNALINVOKER,
  '__module__' : 'qwak.execution.v1.execution_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.execution.v1.ExternalInvoker)
  })
_sym_db.RegisterMessage(ExternalInvoker)

InternalInvoker = _reflection.GeneratedProtocolMessageType('InternalInvoker', (_message.Message,), {
  'DESCRIPTOR' : _INTERNALINVOKER,
  '__module__' : 'qwak.execution.v1.execution_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.execution.v1.InternalInvoker)
  })
_sym_db.RegisterMessage(InternalInvoker)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n%com.qwak.ai.features.execution.api.v1P\001Z#qwak/execution_management;execution'
  _EXECUTIONSTATUS._serialized_start=1925
  _EXECUTIONSTATUS._serialized_end=2256
  _EXECUTIONENTRY._serialized_start=285
  _EXECUTIONENTRY._serialized_end=1579
  _EXECUTIONDEPLOYMENTDETAILS._serialized_start=1581
  _EXECUTIONDEPLOYMENTDETAILS._serialized_end=1670
  _INVOKERDETAILS._serialized_start=1673
  _INVOKERDETAILS._serialized_end=1861
  _EXTERNALINVOKER._serialized_start=1863
  _EXTERNALINVOKER._serialized_end=1903
  _INTERNALINVOKER._serialized_start=1905
  _INTERNALINVOKER._serialized_end=1922
# @@protoc_insertion_point(module_scope)



import functools
from collections.abc import Iterable
from dataclasses import dataclass, field
from typing import Any, Callable, Optional


@dataclass
class ConfigCliMap:
    """Mapping between cli argument to properties argument.

    Attributes:
        key (str): key of the argument in the cli.
        prop (str): property path as nested json object (obj.prop1.prop2)
        validation_func (Callable): function which accept two parameters value (value to validate and is_required param.
        is_required (bool): True if value is required else False.
    """

    key: str
    prop: str
    validation_func: Callable = field(default=lambda: True)
    is_required: bool = False


def validate_list_of_strings(value: Any, is_required: bool) -> bool:
    """Validate if a list include only strings

    Args:
        value: Value to perform validation on.
        is_required: True if value is required.

    Returns:
        bool: True if value is valid.

    Examples:
        >>> validate_bool(["1", "3"], False)
        True
        >>> validate_bool(["1", 3], False)
        False
        >>> validate_bool(None, True)
        False
    """
    if not value and not is_required:
        return True
    elif not isinstance(value, Iterable):
        return False

    return all(isinstance(p, str) for p in value)


def validate_string(value: Any, is_required: bool) -> bool:
    """Validate if value is a string.

    Args:
        value: Value to perform validation on.
        is_required: True if value is required.

    Returns:
        bool: True if value is valid.

    Examples:
        >>> validate_string("3", False)
        True
        >>> validate_string(3, False)
        False
        >>> validate_string(None, True)
        False
    """
    if value:
        return isinstance(value, str)
    elif is_required:
        return False

    return True


def validate_float(value: Any, is_required: bool) -> bool:
    """Validate if value is a float.

    Args:
        value: Value to perform validation on.
        is_required: True if value is required.

    Returns:
        bool: True if value is valid.

    Examples:
        >>> validate_float(3, True)
        True
        >>> validate_float(3.5, True)
        True
        >>> validate_float(None, True)
        False
    """
    if value:
        return isinstance(value, (float, int))
    elif is_required:
        return False

    return True


def validate_int(value: Any, is_required: bool) -> bool:
    """Validate if value is a int.

    Args:
        value: Value to perform validation on.
        is_required: True if value is required.

    Returns:
        bool: True if value is valid.

    Examples:
        >>> validate_int(3, True)
        True
        >>> validate_int(3.5, True)
        False
        >>> validate_int(None, True)
        False
    """
    if value:
        return isinstance(value, int) or (isinstance(value, str) and value.isdigit())
    elif is_required:
        return False

    return True


def validate_positive_int(value: Any, is_required: bool) -> bool:
    """Validate if value is a positive int.

    Args:
        value: Value to perform validation on.
        is_required: True if value is required.

    Returns:
        bool: True if value is valid.

    Examples:
        >>> validate_positive_int(3, True)
        True
        >>> validate_positive_int(3.5, True)
        False
        >>> validate_positive_int(None, True)
        False
        >>> validate_positive_int(-1, True)
        False
    """
    if value:
        if isinstance(value, int) or (isinstance(value, str) and value.isdigit()):
            return int(value) > 0
        else:
            return False
    elif is_required:
        return False

    return True


def validate_bool(value: Any, is_required: bool) -> bool:
    """Validate if value is a boolean.

    Args:
        value: Value to perform validation on.
        is_required: True if value is required.

    Returns:
        bool: True if value is valid.

    Examples:
        >>> validate_bool(True, False)
        True
        >>> validate_bool("true", False)
        False
        >>> validate_bool(None, True)
        False
    """
    if value is not None:
        return isinstance(value, bool)
    elif is_required:
        return False

    return True


def validate_enum(enum: Any) -> Callable:
    """Create function to validate if value in enum.

    Args:
        enum: enum to validate on

    Returns:
        bool: Callable.
    """

    def validate(value: Any, is_required: bool) -> bool:
        if value and isinstance(value, str):
            return value.upper() in enum.__members__
        elif is_required:
            return False

        return True

    return validate


def validate_variation(value: Any) -> bool:
    """
    Args:
        value: alue to perform validation on.

    Returns:
        bool: True if value is valid.
    """
    if not (hasattr(value, "name") and isinstance(value.name, str)):
        return False

    if not hasattr(value, "traffic"):
        return False

    if not (
        hasattr(value.traffic, "percentage")
        and isinstance(value.traffic.percentage, int)
    ):
        return False

    if not (
        hasattr(value.traffic, "shadow") and isinstance(value.traffic.shadow, bool)
    ):
        return False

    return True


def validate_variations(value: Any, is_required: bool) -> bool:
    """Validate if value is a variation.

    Args:
        value: Value to perform validation on.

    Returns:
        bool: True if value is valid.

    Examples:
        >>> validate_variations([DeployConfig.Realtime.VariationConfig(name='default', traffic=DeployConfig.Realtime.VariationConfig.TrafficConfig(percentage=100, shadow=False))], False)
        True
        >>> validate_variations(["{variation.name: Test}"], False)
        False
        >>> validate_variations(None, True)
        False
    """
    if not value and not is_required:
        return True
    elif not isinstance(value, Iterable):
        return False

    return all(validate_variation(variation) for variation in value)


def validate_purchase_option(purchase_option: Optional[str], is_required: bool):
    """Validate purchase option strings

    Args:
        purchase_option: the purchase option to validate.
        is_required: True if value is required.

    Returns:
        bool: True if value is valid.

    Examples:
        >>> validate_purchase_option("spot", False)
        True
        >>> validate_purchase_option("ondemand", False)
        True
        >>> validate_purchase_option(None, False)
        True
        >>> validate_purchase_option("Ondemand", False)
        False
    """
    if not purchase_option and not is_required:
        return True
    if purchase_option and (
        not validate_string(purchase_option, False)
        or purchase_option
        not in [
            "ondemand",
            "spot",
        ]
    ):
        return False
    return True


def rsetattr(obj: object, attr: str, val: Any):
    """Set attribute in object recursively.

    Args:
        obj: Object to set value in.
        attr: Nested attributes path to value. (obj.prop1.prop2)
        val: Value to set in property
    """
    pre, _, post = attr.rpartition(".")
    setattr(rgetattr(obj, pre) if pre else obj, post, val)


def rgetattr(obj: object, attr: str, *args) -> Any:
    """Get attribute in object recursively.

    Args:
        obj: Object to set value in.
        attr: Nested attributes path to value. (obj.prop1.prop2)
    """

    def _getattr(obj, attr):
        return getattr(obj, attr, *args)

    return functools.reduce(_getattr, [obj] + attr.split("."))



from dataclasses import dataclass, field
from datetime import datetime

from _qwak_proto.qwak.projects.projects_pb2 import (
    Project as ProjectProto,
    ProjectSpec as ProjectSpecProto,
)
from google.protobuf.timestamp_pb2 import Timestamp


@dataclass
class Project:
    project_id: str = field(default=None)
    project_name: str = field(default=None)
    project_description: str = field(default=None)
    models_count: int = field(default=0)
    models_active: int = field(default=0)
    created_by: str = field(default="")
    created_at: datetime = field(default=datetime.now())
    last_modified_by: str = field(default="")
    last_modified_at: datetime = field(default=datetime.now())

    def to_proto(self):
        timestamp = Timestamp()
        return ProjectProto(
            spec=ProjectSpecProto(
                project_id=self.project_id,
                project_name=self.project_name,
                project_description=self.project_description,
                models_count=self.models_count,
                models_active=self.models_active,
                created_by=self.created_by,
                created_at=timestamp.FromDatetime(self.created_at),
                last_modified_by=self.last_modified_by,
                last_modified_at=timestamp.FromDatetime(self.last_modified_at),
            )
        )

    @staticmethod
    def from_proto(project_spec_proto: ProjectSpecProto):
        return Project(
            project_id=project_spec_proto.project_id,
            project_name=project_spec_proto.project_name,
            project_description=project_spec_proto.project_description,
            models_count=project_spec_proto.models_count,
            models_active=project_spec_proto.models_active,
            created_by=project_spec_proto.created_by,
            created_at=datetime.fromtimestamp(
                project_spec_proto.created_at.seconds
                + project_spec_proto.created_at.nanos / 1e9
            ),
            last_modified_by=project_spec_proto.last_modified_by,
            last_modified_at=datetime.fromtimestamp(
                project_spec_proto.last_modified_at.seconds
                + project_spec_proto.last_modified_at.nanos / 1e9
            ),
        )
